resource "aws_s3_bucket" "remote_state" {
  bucket = "${var.name}"
  acl    = "private"

  versioning {
    enabled    = true
    mfa_delete = true
  }
  replication_configuration {
  role = "${aws_iam_role.replication.arn}"

    rules {
      id        = "${var.name}-replication"
      priority  = "1"
      status    = "Enabled"

      destination {
        account_id    = "${var.replication_destination_account_id}"
        bucket        = "${var.replication_destination_bucket_arn}"
      }
    }
  }
}

resource "aws_iam_role" "replication" {
  count =  "${var.cross_account_replication_enabled == "true" ? 1 : 0}"
  name = "${var.name}-replication"

  assume_role_policy = <<POLICY
{
  "Version": "2012-10-17",
  "Statement": [
    {
      "Action": "sts:AssumeRole",
      "Principal": {
        "Service": "s3.amazonaws.com"
      },
      "Effect": "Allow",
      "Sid": ""
    }
  ]
}
POLICY
}

resource "aws_iam_policy" "replication" {
  count = "${var.cross_account_replication_enabled == "true" ? 1 : 0}"
  name = "module-remote-state-iam-role-replication"

  policy = <<POLICY
{
    "Version": "2012-10-17",
    "Statement": [
        {
            "Action": [
                "s3:Get*",
                "s3:ListBucket"
            ],
            "Effect": "Allow",
            "Resource": [
                "arn:aws:s3:::{var.name}",
                "arn:aws:s3:::{var.name}/*"
            ]
        },
        {
            "Action": [
                "s3:ReplicateObject",
                "s3:ReplicateDelete",
                "s3:ReplicateTags",
                "s3:GetObjectVersionTagging"
            ],
            "Effect": "Allow",
            "Resource": "arn:aws:s3:::{var.name}-back-up-prod-tf-state/*"
        }
    ]
}
POLICY
}

resource "aws_iam_policy_attachment" "remote_state_replication" {
  name       = "${var.name}-replication-attachment"
  roles      = ["${aws_iam_role.replication.name}"]
  policy_arn = "${aws_iam_policy.replication.arn}"
}

data "aws_iam_policy_document" "remote_state_always_enc" {
  statement {
    sid       = "DenyIncorrectEncryptionHeader"
    effect    = "Deny"
    actions   = ["s3:PutObject"]
    resources = ["arn:aws:s3:::${var.name}/*"]

    principals {
      type        = "AWS"
      identifiers = ["*"]
    }

    condition {
      test     = "StringNotEquals"
      variable = "s3:x-amz-server-side-encryption"
      values   = ["AES256"]
    }
  }

  statement {
    sid       = "DenyUnEncryptedObjectUploads"
    effect    = "Deny"
    actions   = ["s3:PutObject"]
    resources = ["arn:aws:s3:::${var.name}/*"]

    principals {
      type        = "AWS"
      identifiers = ["*"]
    }

    condition {
      test     = "Null"
      variable = "s3:x-amz-server-side-encryption"
      values   = ["true"]
    }
  }
}

resource "aws_s3_bucket_policy" "remote_state" {
  bucket = "${aws_s3_bucket.remote_state.id}"
  policy = "${data.aws_iam_policy_document.remote_state_always_enc.json}"
}
